﻿/*
 * COMMON.c
 *
 * Created: 2013/10/10 21:37:49
 *  Author: Koichi Nishida
 */ 
/*
Copyright (C) 2016 Koichi NISHIDA
email to Koichi NISHIDA: tulip-house@msf.biglobe.ne.jp

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 3 of the License.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <string.h>
#include "BUFFER.h"
#include "COMMON.h"
#include "SD.h"
#include "LCD.h"
#include "DISK2.h"
#include "SMART.h"

// mode : DISK2MODE | SMARTMODE
enum MODE mode;

// whether FlashAir or not
uint8_t isFlashAir = 0;

// decremented toward 0 by a 0.01 sec counter.
volatile uint16_t timerOneShot, timerOneShot2;
// blink at 0.33 sec
volatile uint8_t timerBlink = 0;

// initialize
void COMMON_init(void)
{
	// LEDs
	PORTE.DIRSET = PIN2_bm;
	PORTE.DIRSET = PIN3_bm;

	// initialize input ports
	// PHASE
	PORTA.DIRCLR = PHASE_bm;
	PORTCFG.MPCMASK = PHASE_bm;
	PORTA.PIN0CTRL = PORT_OPC_PULLUP_gc;
	// WDAT	
	PORTA.DIRCLR = PIN6_bm;
	PORTA.PIN6CTRL = PORT_OPC_PULLUP_gc;
	// WREQ
	PORTD.DIRCLR = PIN2_bm;
	PORTD.PIN2CTRL = PORT_OPC_PULLUP_gc;
	// EN2
	PORTD.DIRCLR = PIN5_bm;
	PORTD.PIN5CTRL = PORT_OPC_PULLUP_gc;
	// EN1
	PORTD.DIRCLR = PIN6_bm;
	PORTD.PIN6CTRL = PORT_OPC_PULLUP_gc;
	
	// HDSEL
	PORTA.DIRCLR = PIN7_bm;
	PORTA.PIN7CTRL = PORT_OPC_PULLUP_gc;

	switch (AIR_TYPE) {
		case AIR_19:
			// DIK35
			PORTD.DIRCLR = PIN4_bm;
			PORTD.PIN4CTRL = PORT_OPC_PULLUP_gc;
			break;
		case AIR_20:
		case AIR_C:
			// INTDRV
			PORTD.OUTCLR = PIN4_bm;
			PORTD.DIRSET = PIN4_bm;
			break;
	}

	// initialize output ports
	// RDR
	PORTD.DIRSET = PIN0_bm;

	// SENSE
	PORTD.DIRSET = SENSER_bm;
	
	// internal drive number
	PORTD.DIRSET = PIN7_bm;

	// VPORT0 = PORTA, VPORT1 = PORTC, VPORT2 = PORTD
	PORTCFG.VPCTRLA = PORTCFG_VP02MAP_PORTA_gc | PORTCFG_VP13MAP_PORTC_gc;
	PORTCFG.VPCTRLB = PORTCFG_VP02MAP_PORTD_gc;

	// PHASE interrupt
	PORTA.PIN0CTRL = PORT_ISC_BOTHEDGES_gc;
	PORTA.PIN1CTRL = PORT_ISC_BOTHEDGES_gc;
	PORTA.PIN2CTRL = PORT_ISC_BOTHEDGES_gc;
	PORTA.PIN3CTRL = PORT_ISC_BOTHEDGES_gc;
	PORTA.INT0MASK = PHASE_bm;
	PMIC.CTRL = PMIC_LOLVLEN_bm; 
	PORTA.INTCTRL = PORT_INT0LVL_LO_gc;

	// TCC0 about 0.01 sec used by UI
	TCC0.PER = 313;
	TCC0.CTRLA = TC0_CLKSEL2_bm|TC0_CLKSEL1_bm|TC0_CLKSEL0_bm;			// prescaler : clk/1024
	TCC0.INTCTRLA = 0;	// meddle level interrupt, but off for the time being
	PMIC.CTRL = PMIC_LOLVLEN_bm;

	if (mode == DISK2MODE)
		for (uint8_t i = 0; i < 2; i++) buffer2.disk2.img[i].valid = 0;
	else
		for (uint8_t i = 0; i < 4; i++) buffer2.smart.img[i].valid = 0;
}

// timer handler
ISR(
	TCC0_OVF_vect
) {
	TCC0_INTFLAGS = 1;
	static uint8_t counter = 0;
	if (timerOneShot>0) timerOneShot--;
	if (timerOneShot2>0) timerOneShot2--;
		
	if (++counter==33) {counter = 0; timerBlink ^= 1;}
}

// convert a '0'-'F' to 0x0 - 0xf
uint8_t to16(uint8_t a)
{
	if (a>='0' && a<='9') return (a-'0');
	if (a>='A' && a<='F') return (a-'A'+10);
	if (a>='a' && a<='f') return (a-'a'+10);
	return 0;	
}

// examine whether SmartPort mode or Disk2 mode at startup.
enum MODE GET_MODE(void)
{
	uint8_t err = 0, eof = 0, bt = 0;
	struct FILE fp;
	FILE_open(&fp, 0, "MODE    ", "TXT", 1, &err);
	if (err) {
		return DISK2MODE;
	} else {
		FILE_seq_reset(&fp);
		FILE_seq_read_byte(&fp, &eof, &bt, &err);
		if (err) return DISK2MODE;
		if (bt=='S') return SMARTMODE;
		return DISK2MODE;
	}
}

// set to SmartPort mode or Disk2 mode.
void SET_MODE(enum MODE m)
{
	uint8_t err = 0;
	struct FILE fp;
	FILE_open(&fp, 0, "MODE    ", "TXT", 1, &err);
	if (err) {
		err = 0;
		FILE_create(0, "MODE    TXT", 1, &err);
		if (err) return;
		FILE_open(&fp, 0, "MODE    ", "TXT", 1, &err);
		if (err) return;
	}
	FILE_seq_reset(&fp);

	FILE_seq_write_byte(&fp, (m==SMARTMODE)?'S':'D', &err);
	if (err) return;
	FILE_seq_write_byte_end(&fp, &err);
}
