﻿/*
 * FLASHAIR.c
 *
 * Created: 2014/10/21 22:42:32
 *  Author: 浩一
 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include <string.h>
#include <util/delay.h>
#include "SD.h"
#include "BUFFER.h"
#include "MOUNT.h"
#include "LCD.h"
#include "COMMON.h"
#include "DISK2.h"
#include "SMART.h"
#include "MESSAGE.h"

// examine whether the card is FlashAir or not.
uint8_t IS_FLASHAIR(void)
{
	uint8_t err = 0;
	struct FILE fp;
	FILE_openAbs(&fp, "SD_WLAN    CONFIG     ", 1, &err);
	return !err;
}

void FLASHAIR_set_status(void)
{
	uint8_t err = 0;

	switch (mode) {
		case DISK2MODE:	{
			SD_writeSharedMemoryBegin(0, 512, &err);
			// 0x0054 = 'T' = status command
			SPI_writeByte('T', &err);
			SPI_writeByte(mode, &err);
			SPI_writeByte(DISK2_ph_track[0]>>2, &err);
			SPI_writeByte(DISK2_ph_track[1]>>2, &err);
			SPI_writeByte(!EN1, &err);
			SPI_writeByte(!EN2, &err);
/*
			switch (AIR_TYPE) {
				case AIR_19:
					SPI_writeByte(0x0, &err);
					break;
				case AIR_20:
					SPI_writeByte(0x80 | ((PORTD.OUT & PIN4_bm)?1:0), &err);
					break;
				case AIR_C:
					SPI_writeByte(0x80 | ((PORTD.OUT & PIN4_bm)?1:0), &err);
					break;
			}
*/
			SPI_writeByte((DISK2_globalProtect[0]?0x10:0)|(DISK2_globalProtect[1]?0x20:0)|((PORTD.OUT & PIN4_bm)?1:0), &err);
			SPI_writeByte((FIRMWRITTEN&1)|((PORTD.OUT&PIN7_bm)?0x2:0), &err);
			for (uint8_t drv=0; drv<2; drv++) {
				switch (DISK2_img_processing[drv]) {
					case 0: {
						struct FILE *imgp = &buffer2.disk2.img[drv];
						if (imgp->valid) {
							for (uint8_t i=0; i<8; i++) SPI_writeByte(imgp->name[i], &err);
						} else {
							for (uint8_t i=0; i<8; i++) SPI_writeByte(' ', &err);
						}
						break;
					}
					case NIC_CONVERTING:
						SPI_writeBytes((uint8_t *)"CONVERT!", 8, &err);	
						break;				
					case WRITING_BACK:
						SPI_writeBytes((uint8_t *)"WR BACK!", 8, &err);
						break;			
				}
			}
#ifdef DEBUG			
			for (uint16_t i=0; i<(64-24); i++) SPI_writeByte(0, &err);
			for (uint16_t i=0; i<64; i++) SPI_writeByte(message[i], &err);
			for (uint16_t i=0; i<(512-128); i++) SPI_writeByte(0, &err);
#else
			for (uint16_t i=0; i<(512-24); i++) SPI_writeByte(0, &err);
#endif			
			SD_writeSharedMemoryEnd(&err);
			break;
		}
		case SMARTMODE: {
			SD_writeSharedMemoryBegin(0, 512, &err);
			// 0x0054 = 'T' = status command
			SPI_writeByte('T', &err);
			SPI_writeByte(mode, &err);
			SPI_writeByte(0, &err);
			SPI_writeByte(0, &err);
			SPI_writeByte(0, &err);
			SPI_writeByte(0, &err);
			SPI_writeByte((SMART_globalProtect[0]?0x10:0)|(SMART_globalProtect[1]?0x20:0)
			|(SMART_globalProtect[2]?0x40:0)|(SMART_globalProtect[3]?0x80:0), &err);
			SPI_writeByte(FIRMWRITTEN, &err);
			for (uint8_t drv=0; drv<4; drv++) {
				struct FILE *imgp = &buffer2.smart.img[drv];
				if (imgp->valid) {
					for (uint8_t i=0; i<8; i++) SPI_writeByte(imgp->name[i], &err);
				} else {
					for (uint8_t i=0; i<8; i++) SPI_writeByte(' ', &err);
				}
			}
#ifdef DEBUG
			for (uint16_t i=0; i<(64-40); i++) SPI_writeByte(0, &err);
			for (uint16_t i=0; i<64; i++) SPI_writeByte(message[i], &err);
			for (uint16_t i=0; i<(512-128); i++) SPI_writeByte(0, &err);
#else
			for (uint16_t i=0; i<(512-40); i++) SPI_writeByte(0, &err);
#endif
			SD_writeSharedMemoryEnd(&err);
			break;
		}
		case FIRMMODE: {
			SD_writeSharedMemoryBegin(0, 512, &err);
			// 0x0054 = 'T' = status command
			SPI_writeByte('T', &err);
			SPI_writeByte(4, &err);
			SPI_writeByte(0, &err);
			SPI_writeByte(0, &err);
			SPI_writeByte(0, &err);
			SPI_writeByte(0, &err);
			SPI_writeByte(0, &err);
			SPI_writeByte(FIRMWRITTEN, &err);
			for (uint8_t drv=0; drv<4; drv++) {
				for (uint8_t i=0; i<8; i++) SPI_writeByte(' ', &err);
			}
			for (uint16_t i=0; i<(512-40); i++) SPI_writeByte(0, &err);
			SD_writeSharedMemoryEnd(&err);
			break;
		}
	}
}

// check and execute a command from shared memory
uint8_t FLASHAIR_command(void)
{
	uint8_t err = 0;
	uint8_t ret = 0;

	// read first word (command)
	SD_readSharedMemoryBegin(0, 512, &err);
	uint8_t cmd = SPI_readByte(&err);
	uint8_t param = SPI_readByte(&err)-'0';
	uint8_t param2 = SPI_readByte(&err)-'0';
	for (uint16_t i=0; i<509; i++) SPI_readByte(&err);
	SD_readSharedMemoryEnd(&err);
	
	switch (cmd) {
		case 'P': {
			switch (mode) {
			case DISK2MODE:
				DISK2_globalProtect[param&0b11] = ((param&0b100)?1:0);
				break;
			case SMARTMODE:
				SMART_globalProtect[param&0b11] = ((param&0b100)?1:0);
				break;
			case FIRMMODE:
				break;
			}
			break;
		}
		case 'I': {
			if ((AIR_TYPE == AIR_C) || (AIR_TYPE == AIR_20)) {
				switch (param) {
					case 0:
						// unisdisk
						cli();
#ifdef AIRDISP
						LCD_locate(0,1);
						LCD_print("UNISDISK", 8);
#endif
						sei();
						PORTD.OUTCLR = PIN4_bm;
						break;
					case 1:
						// real drive
						cli();
#ifdef AIRDISP
						LCD_locate(0,1);
						LCD_print("REAL DRV", 8);
#endif
						sei();
						PORTD.OUTSET = PIN4_bm;
						break;
				}
				if (param2) {
					PORTD.OUTSET = PIN7_bm;
				} else {
					PORTD.OUTCLR = PIN7_bm;
				}
			}
			break;
		}
		// drive change command
		case 'D': {
			FILE_getShortFileName(buffer2.wifi.short_name, &err);
			if (*(char *)buffer2.wifi.short_name) MOUNT_mount(param/*drive*/);
			ret = 1;
			break;
		}
		// unmount command
		case 'U': {
			strcpy((char *)buffer2.wifi.short_name, "           ");
			MOUNT_mount(param/*drive*/);
			ret = 1;
			break;	
		}
		// write back command
		case 'W': {
			// write back to .dsk file
			MOUNT_writeBack(param/*drive*/);
			break;
		}		
		// reset command
		case 'R': {
			FLASHAIR_set_status();
			cli();
#ifdef AIRDISP
			LCD_locate(0,0);
#endif
			switch (param/*drive*/) {
				case 0:
					SET_MODE(DISK2MODE);
#ifdef AIRDISP
					LCD_printAll("RESET asDISK2Drv");
#endif
					break;
				case 1:
					SET_MODE(SMARTMODE);
#ifdef AIRDISP
					LCD_printAll("RESET asSmartDrv");
#endif
					break;
			}
			_delay_ms(100);
			// RESET !
			CPU_CCP=CCP_IOREG_gc;
			RST.CTRL=RST_SWRST_bm;
			while (1);
		}
	}
	FLASHAIR_set_status();
	return ret;
}